@@ -19,9 +19,10 @@ DLF = sys.stdout # Default Log File
 
 ET = None
 ET_has_iterparse = False
+Element_has_iter = False
 
 def ensure_elementtree_imported(verbosity, logfile):
-    global ET, ET_has_iterparse
+    global ET, ET_has_iterparse, Element_has_iter
     if ET is not None:
         return
     if "IronPython" in sys.version:
@@ -47,6 +48,7 @@ def ensure_elementtree_imported(verbosity, logfile):
             ET_has_iterparse = True
         except NotImplementedError:
             pass
+    Element_has_iter = hasattr(ET.ElementTree, 'iter')
     if verbosity:
         etree_version = repr([
             (item, getattr(ET, item))
@@ -225,14 +227,15 @@ def make_name_access_maps(bk):
                 if bk.verbosity:
                     print(msg, file=bk.logfile)
         name_and_scope_map[key] = nobj
+        sort_data = (nobj.scope, namex, nobj)
         if name_lcase in name_map:
-            name_map[name_lcase].append((nobj.scope, nobj))
+            name_map[name_lcase].append(sort_data)
         else:
-            name_map[name_lcase] = [(nobj.scope, nobj)]
+            name_map[name_lcase] = [sort_data]
     for key in name_map.keys():
         alist = name_map[key]
         alist.sort()
-        name_map[key] = [x[1] for x in alist]
+        name_map[key] = [x[2] for x in alist]
     bk.name_and_scope_map = name_and_scope_map
     bk.name_map = name_map
 
@@ -243,7 +246,7 @@ class X12General(object):
             fprintf(self.logfile, "\n=== %s ===\n", heading)
         self.tree = ET.parse(stream)
         getmethod = self.tag2meth.get
-        for elem in self.tree.getiterator():
+        for elem in self.tree.iter() if Element_has_iter else self.tree.getiterator():
             if self.verbosity >= 3:
                 self.dump_elem(elem)
             meth = getmethod(elem.tag)
@@ -289,7 +292,7 @@ class X12Book(X12General):
         self.tree = ET.parse(stream)
         getmenu = self.core_props_menu.get
         props = {}
-        for elem in self.tree.getiterator():
+        for elem in self.tree.iter() if Element_has_iter else self.tree.getiterator():
             if self.verbosity >= 3:
                 self.dump_elem(elem)
             menu = getmenu(elem.tag)
@@ -363,7 +366,14 @@ class X12Book(X12General):
             if self.verbosity >= 2:
                 self.dumpout('Ignoring sheet of type %r (name=%r)', reltype, name)
             return
-        bk._sheet_visibility.append(True)
+        state = elem.get('state')
+        visibility_map = {
+            None: 0,
+            'visible': 0,
+            'hidden': 1,
+            'veryHidden': 2
+            }
+        bk._sheet_visibility.append(visibility_map[state])
         sheet = Sheet(bk, position=None, name=name, number=sheetx)
         sheet.utter_max_rows = X12_MAX_ROWS
         sheet.utter_max_cols = X12_MAX_COLS
@@ -504,6 +514,7 @@ class X12Sheet(X12General):
         self.rowx = -1 # We may need to count them.
         self.bk = sheet.book
         self.sst = self.bk._sharedstrings
+        self.merged_cells = sheet.merged_cells
         self.warned_no_cell_name = 0
         self.warned_no_row_num = 0
         if ET_has_iterparse:
@@ -521,8 +532,34 @@ class X12Sheet(X12General):
                 elem.clear() # destroy all child elements (cells)
             elif elem.tag == U_SSML12 + "dimension":
                 self.do_dimension(elem)
+            elif elem.tag == U_SSML12 + "mergeCell":
+                self.do_merge_cell(elem)
         self.finish_off()
-        
+
+    def process_comments_stream(self, stream):
+        root = ET.parse(stream).getroot()
+        author_list = root[0]
+        assert author_list.tag == U_SSML12 + 'authors'
+        authors = [elem.text for elem in author_list]
+        comment_list = root[1]
+        assert comment_list.tag == U_SSML12 + 'commentList'
+        cell_note_map = self.sheet.cell_note_map
+        from .sheet import Note
+        text_tag = U_SSML12 + 'text'
+        r_tag = U_SSML12 + 'r'
+        t_tag = U_SSML12 + 't'
+        for elem in comment_list.findall(U_SSML12 + 'comment'):
+            ts = elem.findall('./' + text_tag + '/' + t_tag)
+            ts += elem.findall('./' + text_tag + '/' + r_tag + '/' + t_tag)
+            ref = elem.get('ref')
+            note = Note()
+            note.author = authors[int(elem.get('authorId'))]
+            note.rowx, note.colx = coords = cell_name_to_rowx_colx(ref)
+            note.text = ''
+            for t in ts:
+                note.text += cooked_text(self, t)
+            cell_note_map[coords] = note
+
     def do_dimension(self, elem):
         ref = elem.get('ref') # example: "A1:Z99" or just "A1"
         if ref:
@@ -532,6 +569,16 @@ class X12Sheet(X12General):
             self.sheet._dimnrows = rowx + 1
             self.sheet._dimncols = colx + 1
 
+    def do_merge_cell(self, elem):
+        # The ref attribute should be a cell range like "B1:D5".
+        ref = elem.get('ref')
+        if ref:
+            first_cell_ref, last_cell_ref = ref.split(':')
+            first_rowx, first_colx = cell_name_to_rowx_colx(first_cell_ref)
+            last_rowx, last_colx = cell_name_to_rowx_colx(last_cell_ref)
+            self.merged_cells.append((first_rowx, last_rowx + 1,
+                                      first_colx, last_colx + 1))
+
     def do_row(self, row_elem):
     
         def bad_child_tag(child_tag):
@@ -569,6 +616,8 @@ class X12Sheet(X12General):
                 try:
                     for c in cell_name:
                         charx += 1
+                        if c == '$':
+                            continue
                         lv = letter_value[c]
                         if lv:
                             colx = colx * 26 + lv
@@ -683,15 +732,6 @@ class X12Sheet(X12General):
         }
     augment_keys(tag2meth, U_SSML12)
 
-def getzflo(zipfile, member_path):
-    # GET a Zipfile File-Like Object for passing to
-    # an XML parser
-    try:
-        return zipfile.open(member_path) # CPython 2.6 onwards
-    except AttributeError:
-        # old way
-        return BYTES_IO(zipfile.read(member_path))
-
 def open_workbook_2007_xml(
     zf,
     component_names,
@@ -718,41 +758,47 @@ def open_workbook_2007_xml(
     bk.ragged_rows = ragged_rows
 
     x12book = X12Book(bk, logfile, verbosity)
-    zflo = getzflo(zf, 'xl/_rels/workbook.xml.rels')
+    zflo = zf.open(component_names['xl/_rels/workbook.xml.rels'])
     x12book.process_rels(zflo)
     del zflo
-    zflo = getzflo(zf, 'xl/workbook.xml')
+    zflo = zf.open(component_names['xl/workbook.xml'])
     x12book.process_stream(zflo, 'Workbook')
     del zflo
-    props_name = 'docProps/core.xml'
+    props_name = 'docprops/core.xml'
     if props_name in component_names:
-        zflo = getzflo(zf, props_name)
+        zflo = zf.open(component_names[props_name])
         x12book.process_coreprops(zflo)
 
     x12sty = X12Styles(bk, logfile, verbosity)
     if 'xl/styles.xml' in component_names:
-        zflo = getzflo(zf, 'xl/styles.xml')
+        zflo = zf.open(component_names['xl/styles.xml'])
         x12sty.process_stream(zflo, 'styles')
         del zflo
     else:
         # seen in MS sample file MergedCells.xlsx
         pass
 
-    sst_fname = 'xl/sharedStrings.xml'
+    sst_fname = 'xl/sharedstrings.xml'
     x12sst = X12SST(bk, logfile, verbosity)
     if sst_fname in component_names:
-        zflo = getzflo(zf, sst_fname)
+        zflo = zf.open(component_names[sst_fname])
         x12sst.process_stream(zflo, 'SST')
         del zflo
 
     for sheetx in range(bk.nsheets):
         fname = x12book.sheet_targets[sheetx]
-        zflo = getzflo(zf, fname)
+        zflo = zf.open(component_names[fname])
         sheet = bk._sheet_list[sheetx]
         x12sheet = X12Sheet(sheet, logfile, verbosity)
         heading = "Sheet %r (sheetx=%d) from %r" % (sheet.name, sheetx, fname)
         x12sheet.process_stream(zflo, heading)
         del zflo
+        comments_fname = 'xl/comments%d.xml' % (sheetx + 1)
+        if comments_fname in component_names:
+            comments_stream = zf.open(component_names[comments_fname])
+            x12sheet.process_comments_stream(comments_stream)
+            del comments_stream
+
         sheet.tidy_dimensions()
 
     return bk
